@@ -807,6 +807,39 @@ sub draw_form_field_string {
 	drawer_call ('draw_form_field_string', @_);
 }
 
+
+################################################################################
+
+=head1 draw_form_field_file
+
+    'file'.   - C<draw_form>.
+
+=head2 
+
+=over
+
+=item name
+
+ CGI-      C<$data>.
+
+=item label
+
+ 
+
+=item size
+
+  C<size>
+
+=back
+
+=cut
+
+################################################################################
+
+sub draw_form_field_file {
+	drawer_call ('draw_form_field_file', @_);
+}
+
 ################################################################################
 
 =head1 draw_form_field_button
